#include <iostream.h>
#include <fstream.h>
#include <ctype.h>
#include "prompt.h"
#include "apstring.h"
#include "apvector.h"

struct jword
{
    apstring info;
    bool isOk;
    jword(const apstring & s="")
      : info(s),
        isOk(true)
    {}
};

class Jotto
{
  public:
    Jotto();
    void load(const apstring & filename);
    void play();
    void clear();
  private:
  	apvector<jword> myList;
  	int               myCount;
  	
  	void process(const apstring & guess,
  		         int common);
    
    int  commonCount(const apstring & a, const apstring & b);
    void getGuess(apstring & word);
  
};


Jotto::Jotto()
  : myList(100),
    myCount(0)
{

}

void Jotto::load(const apstring & filename)
{
    ifstream input(filename.c_str());
    apstring word;
    if (input.fail())
    {
		cout << "couldn't open data file " << filename << endl;
		exit(1);
    }
    
    while (getline(input,word))              // read one line
    {
		word = word.substr(0,5);     // truncate # letters
		if (isalpha(word[0]) && islower(word[0])) // begins with alphabetic char?
		{
		    if (myCount >= myList.length())
		    {
		        myList.resize(myCount * 2);
		    }
		    myList[myCount] = word;
		    myCount++;
		}
    }
}

void Jotto::clear()
{
	int k;
	for(k=0; k < myCount; k++)
	{
		myList[k].isOk = true;
	}
}

void Jotto::getGuess(apstring & word)
{
    int k;
    for(k=0; k <myCount; k++)
    {
        if (myList[k].isOk)
        {
            word = myList[k].info;
            myList[k].isOk = false;
            return;
        }
    }
    word = "";
}

void Jotto::play()
{
    apstring guess,answer;
    int common;
    int count = 1;
    clear();
    getGuess(guess);
    
    while (guess != "")
    {
        cout << "I guess " << guess << endl;
       
        
        if (PromptYesNo("is that your word? "))
        {
        	cout << "I'm a winner! in " << count << " guesses" << endl;
            break;
        }
        else
        {
	        cout << "how many letters in common with your word: ";
	        cin >> common;
	        
	        process(guess,common);
        } 
        getGuess(guess);
        count++;
    }
}

void Jotto::process(const apstring & guess,
  		            int common)
{
    int k;
    for(k=0; k < myCount; k++)
    {
        if (myList[k].isOk && 
            commonCount(myList[k].info,guess) != common)
        {
            myList[k].isOk = false;
        }
    }
} 		     


int Jotto::commonCount(const apstring & a, const apstring & b)
{
    int count = 0;
    apstring copy = a;
    
    int j,k;
    int len = b.length();
    for(k=0; k < len; k++)
    {
        for(j=0; j < len; j++)
        {
            if (copy[j] == b[k])
            {
                copy[j] = '*';
                count++;
                break;
            }
        }
    }
    return count;
}

int main()
{
    apstring file = PromptString("enter jotto file: ");
    Jotto game;
    
    game.load(file);
    
    do
    {
        game.play();
    } while(PromptYesNo("play again?"));
    
    return 0;
}