// see search.h

template <class Kind>
int search(const apvector<Kind> & a, int first, int last,
           const Kind & key)
// sequential search of an apvector          
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found
{
    int k;
    for(k=first; k <= last; k++)
    {
    	if (a[k] == key) return k;
    }
    return npos;
}


template <class Kind>
int bsearch(const apvector<Kind> & a, int first, int last,
            const Kind & key)
// binary search of an apvector
// precondition: a[first] <= ... <= a[last]            
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found
{
	int k;
	int mid;
	
	while (first <= last)
	{
		mid = (first + last)/2;
		
		if      (a[mid] < key) first = mid + 1;
		else if (key < a[mid]) last  = mid - 1;
		else /* equals key */  return mid;
	}
	return npos;
}

template <class Kind, class Comparer>
int search(const apvector<Kind> & a, int first, int last,
           const Kind & key, const Comparer & comp)
// sequential search of an apvector          
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found
{
    int k;
    for(k=first; k <= last; k++)
    {
    	if (comp.compare(a[k],key) == 0) return k;
    }
    return npos;
}

template <class Kind, class Comparer>
int bsearch(const apvector<Kind> & a, int first, int last,
            const Kind & key, const Comparer & comp)
// binary search of an apvector
// precondition: a[first] <= ... <= a[last]            
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found
{
	int k;
	int mid;
	
	while (first <= last)
	{
		mid = (first + last)/2;
		
		if      (comp.compare(a[mid],key) < 0) first = mid + 1;
		else if (comp.compare(key,a[mid]) < 0) last  = mid - 1;
		else    /* equals key */               return mid;
	}
	return npos;
}

