#ifndef _SEARCH_H
#define _SEARCH_H

#include "apvector.h"
#include "comparer.h"

// ********************************
//
// sequential and binary search
// the templated type Kind must support
// comparison by == for search and using < for binary search
//
// ********************************

extern const int npos;   // defined in aplibrary, apstring.cpp

template <class Kind>
int search(const apvector<Kind> & a, int first, int last,
           const Kind & key);
// sequential search of an apvector          
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found


template <class Kind>
int bsearch(const apvector<Kind> & a, int first, int last,
            const Kind & key);
// binary search of an apvector
// precondition: a[first] <= ... <= a[last]            
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found


template <class Kind, class Comparer>
int search(const apvector<Kind> & a, int first, int last,
           const Kind & key, const Comparer & comp);
// sequential search of an apvector          
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found


template <class Kind, class Comparer>
int bsearch(const apvector<Kind> & a, int first, int last,
            const Kind & key, const Comparer & comp);
// binary search of an apvector
// precondition: a[first] <= ... <= a[last]            
// postcondition: returns index of first occurrence of key in
//                a[first]..a[last]
//                returns npos if key not found


#include "search.cpp"

#endif