#include <iostream.h>
#include <iomanip.h>         // for width()
#include <fstream.h>         // for ifstream/ofstream
#include <ctype.h>           // for tolower
#include "apstring.h"
#include "apvector.h"
#include "sortall.h"

// **********************************
// author: Owen Astrachan
//
// see people.cpp for comments, this version sorts
// using different criteria and the functions in sortall.h
//
// **********************************

struct Person
{
	apstring myFirst,myLast;
	apstring mySchool;
	apstring myGender;
	int myNumber;
	
	Person();
	Person(const apstring & first,
	       const apstring & last,
	       const apstring & school,
	       const apstring & gender,
	       int number);
	       
	void Print(ostream & out)     const;
	bool Less(const Person & rhs) const;
	   
};

ostream& operator << (ostream & out, const Person & p)
//postcondition: p printed to out, out returned
{
	p.Print(out);
	return out;
}

bool operator < (const Person & lhs, const Person & rhs)
{
	return lhs.Less(rhs);
}

bool operator > (const Person & lhs, const Person & rhs)
{
    return rhs < lhs;
}

bool operator == (const Person & lhs, const Person & rhs)
{
   return  (! (rhs < lhs)) && (! (lhs < rhs));
}

// ************************************
//             implement Person
//  ***********************************

Person::Person()
// postcondition: initialized with default values
{

}

Person::Person(const apstring & first,
	       	   const apstring & last,
	           const apstring & school,
	           const apstring & gender,
	           int number)
    : myFirst(first),
      myLast(last),
      mySchool(school),
      myGender(gender),
      myNumber(number)
{
	// initialized using initializer list
}      	           

void Person::Print(ostream & out) const
// postcondition: person printed to stream out
{
    out << setw(20) << myFirst + " " + myLast
        << setw(15) << mySchool << char(toupper(myGender[0])) << " "
        << myNumber;
}

bool Person::Less(const Person & rhs) const
{
    if (myLast < rhs.myLast) return true;
    if (myLast > rhs.myLast) return false;
    
    return myFirst < rhs.myFirst;
}

// **********************************
// Collection of people
// **********************************


class Collection
{
  public:
  
    Collection();
    
    // accessor or const functions
       
    enum Order {byName, byNumber};
    
    void Print(ostream & out)       const;  // print collection
    
    // mutator functions
    
    void Add(const Person & p);   // add p to collection
    void Clear();                 // remove everyone
    void Sort();                  // sort 
	
  private:
  
      apvector<Person> myList;    // the people in collection
      int              myCount;   // # people in collection
};

Collection::Collection()
  : myList(10),
    myCount(0)
/// postcondition: collection initialized with zero people
{

}


void Collection::Print(ostream & out) const
// postcondition: all elements of collection are printed
{
    int k;
    for(k=0;k < myCount; k++)
    {
    	out << myList[k] << endl;
    }
    
    cout << "------ " << myCount << " total people" << endl;
}

void Collection::Add(const Person & p)
// postcondition: p added to collection
{
    if (myCount >= myList.length())
    {
        myList.resize(myList.length()*2);
    }
    myList[myCount] = p;
    myCount++;
}

void Collection::Sort()
{
    SelectSort(myList,myCount);
}

void Collection::Clear()
{
	myCount = 0;
}

int main()
{
	apstring filename = "c:\\ets\\berkworkshop\\98participants";
	ifstream input;
	
	input.open(filename.c_str());
	if (! input.good())
	{
		cerr << "could not open " << filename << endl;
		return 1;
	}
	
	
	Collection group,matchGroup;
	apstring name,gender,school,number;
    int spaceIndex;
    
    cout << "press return" << endl;
	getline(cin,name);
    
    while (getline(input,name)   &&
	       getline(input,gender) &&
	       getline(input,school) &&
	       getline(input,number))
    {
        spaceIndex = name.find(' ');
        Person p(name.substr(0,spaceIndex),
                 name.substr(spaceIndex+1, name.length()),
                 school,
                 gender,
                 atoi(number.c_str()));
                 
        group.Add(p);
    }
	
	group.Print(cout);
	
	cout << endl << "sorted by name" << endl << endl;
	
	cout << "press return" << endl;
	getline(cin,name);
	
	group.Sort();
	group.Print(cout);
	
    return 0;	       
}
