//  aplist.cpp
//
//  DRAFT
//
//  Chris Nevison, Mark Weiss
//  February 2, 1998
//
//  An implementation of a two-way list with iterator
//
//  This implementation of aplist uses a doubly linked list with
//  a header node and a tail node which do not contain items
//  stored on the list.

#include <process.h>  // to include the abort function

///////////////////////// aplistNode /////////////////////////////////
template<class ItemType>
aplistNode<ItemType>::aplistNode(const ItemType & item, 
                                 aplistNode<ItemType> * prev, 
                                 aplistNode<ItemType> * next)

    : myItem(item),
      myPrev(prev),
      myNext(next)
{}


/////////////////////////// aplist /////////////////////////////////

template<class ItemType>
aplist<ItemType>::aplist()                  // default constructor
{                                           // creates empty list
  ItemType item;
  myHead = new aplistNode<ItemType>(item,NULL, NULL);
  myTail  = new aplistNode<ItemType>(item, myHead, NULL);
  myHead->myNext = myTail;
}


template<class ItemType>
aplist<ItemType>::aplist(const aplist<ItemType> & rhs) // copy constructor
{                                                      
  myHead = new aplistNode<ItemType>(rhs.myHead->myItem, NULL, NULL);
  myTail = new aplistNode<ItemType>(rhs.myTail->myItem, myHead, NULL);
  myHead->myNext = myTail;
  *this = rhs;                    // use operator= to copy all nodes
}


template<class ItemType>
aplist<ItemType>::~aplist()       // destructor, must destroy all nodes           
{
  MakeEmpty();
  delete myHead;
  delete myTail;
}

template<class ItemType>
const aplist<ItemType> & aplist<ItemType>::operator = 
                                            (const aplist<ItemType> & rhs)
{
  aplistIterator<ItemType> itr(rhs);
  if (this != &rhs){
    MakeEmpty();
    for(itr = rhs.First(); itr.OnList(); itr.Advance())
      Append(itr.Current());
  }
  return *this;
}

template<class ItemType>
bool aplist<ItemType>::IsEmpty() const
{
  return (myHead->myNext == myTail);
}
  
template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::First() const
{
  return NewIterator(myHead->myNext);
}

template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::Last() const  
{
  return NewIterator(myTail->myPrev);
}

template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::FrontMarker() const  
{
  return NewIterator(myHead);
}

template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::EndMarker() const 
{
  return NewIterator(myTail);
}

template<class ItemType>
void  aplist<ItemType>::MakeEmpty()
{
  while(!IsEmpty())
    Erase(First());
}

template<class ItemType>
void  aplist<ItemType>::Append(const ItemType & item)
{
  InsertBefore(EndMarker(), item);
}

template<class ItemType>
void  aplist<ItemType>::Prepend(const ItemType & item)
{
  InsertAfter(FrontMarker(), item);
}

template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::InsertBefore(const aplistIterator<ItemType> & itr,
                               const ItemType & item)
// pre:  itr is an iterator referencing a position on the list or the
//       position of EndMarker
// post: item has been inserted into the list at a position before the
//       one referenced by itr and the position of item is returned
{
  aplistNode<ItemType> * temp;
  if (this != &(itr.myList)){
    cerr << "Attempt to use iterator on wrong list in InsertBefore." << endl;
    abort();
  }
  else if(itr.OnList() || (itr == EndMarker())){
    temp = new aplistNode<ItemType>(item, itr.myPos->myPrev, itr.myPos);
    itr.myPos->myPrev->myNext = temp;
    itr.myPos->myPrev = temp;
  }
  else{
    cerr << "Attempt to InsertBefore when not on list or EndMarker." << endl;
    abort();
  }
  return NewIterator(temp);
}

template<class ItemType>
aplistIterator<ItemType>
aplist<ItemType>::InsertAfter(const aplistIterator<ItemType> & itr,
                              const ItemType & item)
// pre:  itr is an iterator referencing a position on the list or the
//       position of FrontMarker
// post: item has been inserted into the list at a position after the
//       one referenced by itr and the position of item is returned
{
  aplistNode<ItemType> * temp;
  if (this != &(itr.myList)){
    cerr << "Attempt to use iterator on wrong list in InsertAfter." << endl;
    abort();
  }
  else if(itr.OnList() || (itr == FrontMarker())){
    temp = new aplistNode<ItemType>(item, itr.myPos, itr.myPos->myNext);
    itr.myPos->myNext->myPrev = temp;
    itr.myPos->myNext = temp;
  }
  else{
    cerr << "Attempt to InsertAfter when not on list or FrontMarker." << endl;
    abort();
  }
  return NewIterator(temp);
}


template<class ItemType>
void aplist<ItemType>::Erase(aplistIterator<ItemType> & itr)
{
  aplistNode<ItemType> * temp;
  if (this != &(itr.myList)){
    cerr << "Attempt to use iterator on wrong list in Erase." << endl;
    abort();
  }
  else if(itr.OnList()){
    temp = itr.myPos;
    itr.myPos->myNext->myPrev = itr.myPos->myPrev;
    itr.myPos->myPrev->myNext = itr.myPos->myNext;
    itr.myPos = itr.myPos->myNext;
    delete temp;
  }
  else{
    cerr << "Attempt to Erase when not on list." << endl;
    abort();
  }
}

template<class ItemType> 
aplistIterator<ItemType> 
aplist<ItemType>::NewIterator(aplistNode<ItemType> * position) const
{
  aplistIterator<ItemType> itr(*this);
  itr.myPos = position;
  return itr;
}

/////////////////////////// aplistIterator ////////////////////////////

template<class ItemType>
aplistIterator<ItemType>::aplistIterator(const aplist<ItemType> & list)
: myList(list),
  myPos(list.myHead->myNext)
{}

template<class ItemType>
aplistIterator<ItemType>::aplistIterator(const aplistIterator<ItemType> & rhs)
: myList(rhs.myList),
  myPos(rhs.myPos)
{}

template<class ItemType>
aplistIterator<ItemType>::~aplistIterator()
{}


template<class ItemType>
const aplistIterator<ItemType> & aplistIterator<ItemType>::operator = 
                                    (const aplistIterator<ItemType> & rhs)
{
  if (this != &rhs){
    if (&myList != &(rhs.myList)){
      cerr << "ListIterator inconsistency in operator=."<< endl;
      abort();
    }
    myPos  = rhs.myPos;
  }
  return *this;
}


template<class ItemType>
bool aplistIterator<ItemType>::OnList() const
{
  return (myPos != NULL)&&
         (myPos != myList.myHead) && (myPos != myList.myTail);
}


template<class ItemType>
const ItemType & aplistIterator<ItemType>::Current() const
{
  if (!OnList()){
    cerr << "Attempt to access off the list in Current (const)" << endl;
    abort();
  }
  return myPos->myItem;
}

template<class ItemType>
void aplistIterator<ItemType>::SetCurrent(const ItemType & item)
{
  if (!OnList()){
    cerr << "Attempt to access off the list in SetCurrent" << endl;
    abort();
  }
  myPos->myItem = item;
}

template<class ItemType>
bool aplistIterator<ItemType>::operator == (const aplistIterator<ItemType> & rhs) const
{
  return ((&myList == &(rhs.myList)) && (myPos == rhs.myPos));
}

template<class ItemType>
bool aplistIterator<ItemType>::operator != (const aplistIterator<ItemType> & rhs) const
{
  return !(*this == rhs);
}

template<class ItemType>
void aplistIterator<ItemType>::Advance()
{
  if (!OnList()){
    cerr << "Attempt to Advance when not on list." << endl;
    abort();
  }
  myPos = myPos->myNext;
}

template<class ItemType>
void aplistIterator<ItemType>::Backup()
{
  if (!OnList()){
    cerr << "Attempt to Backup when not on list." << endl;
    abort();
  }
  myPos = myPos->myPrev;
}

