// priorityqueue.cpp
//
//
//              !!!  DRAFT   !!!
//
// Chris Nevison
// March 15, 1998
//
// This file specifies a priority queue as a template collection class
//
// The items stored are stored in the priority queue with a priority
// specified when they are stored.  The priority must have the 
// operators ==, !=, < and > defined.  The item with the smallest (first)
// priority is always the next item to be accessed or removed by
// Min and RemoveMin.


template<class ItemType, class PriorityType>
PriorityQueue<ItemType, PriorityType>::
PriorityQueue()
: myHeap(4), myNumItems(0)
{}

template<class ItemType, class PriorityType>
PriorityQueue<ItemType, PriorityType>::
PriorityQueue(const PriorityQueue<ItemType, PriorityType> & pq)
: myHeap(pq.myHeap), myNumItems(pq.myNumItems)
{}

template<class ItemType, class PriorityType>
const PriorityQueue<ItemType, PriorityType> &
PriorityQueue<ItemType, PriorityType>::
operator = (const PriorityQueue<ItemType, PriorityType> & rhs)
{
  if(this != &rhs){
    myHeap = rhs.myHeap;
    myNumItems = rhs.myNumItems;
  }
  return *this;
}

template<class ItemType, class PriorityType>
const ItemType & PriorityQueue<ItemType, PriorityType>::Min()
{
  if(IsEmpty()){
    cerr << "Attempt to access min of empty priority queue." << endl;
    abort();
    return myHeap[0].item;
  }
  else
    return myHeap[0].item;
}

template<class ItemType, class PriorityType>
bool  PriorityQueue<ItemType, PriorityType>::
IsEmpty()
{
  return myNumItems == 0;
}
   
template<class ItemType, class PriorityType>
void PriorityQueue<ItemType, PriorityType>::
RemoveMin()                 
{
  int pos = 0;
  int childpos;
  myNumItems--;
  PriorityType priLast(myHeap[myNumItems].pri);
  
  while(pos <= myNumItems){
    childpos = 2 * pos + 1;
    if(childpos < myNumItems){
      if((childpos + 1 < myNumItems) && 
         (myHeap[childpos + 1].pri < myHeap[childpos].pri))
        childpos++;
      if(priLast < myHeap[childpos].pri){
        myHeap[pos] = myHeap[myNumItems];
        return;
      }
      else{
        myHeap[pos] = myHeap[childpos];
        pos = childpos;
      }
    }
    else{
      myHeap[pos] = myHeap[myNumItems];
      return;
    }
  }
  cerr << "Error in RemoveMin. "<< endl;
  abort();
}

template<class ItemType, class PriorityType>
void PriorityQueue<ItemType, PriorityType>::
RemoveMin(ItemType & item)
{
  item = myHeap[0].item;
  RemoveMin();
}

template<class ItemType, class PriorityType>
void PriorityQueue<ItemType, PriorityType>::
Add(const ItemType & item, const PriorityType & pri)
{
  PQPair<ItemType, PriorityType> p;
  int pos;
  p.item = item;
  p.pri  = pri;

  if(myNumItems == myHeap.length())
    myHeap.resize(myHeap.length() * 2);

  pos = myNumItems;
  myNumItems++;
  while((pos > 0) && (p.pri < myHeap[(pos-1)/2].pri)){
    myHeap[pos] = myHeap[(pos-1)/2];
    pos = (pos-1)/2;
  }
  myHeap[pos] = p;
}

