
// priorityqueue.h
//
//               !!!   DRAFT    !!!
//
// Chris Nevison
// March 15, 1998
//
// This file specifies a priority queue as a template collection class
//
// The items stored are stored in the priority queue with a priority
// specified when they are stored.  The priority must have the 
// operators ==, !=, < and > defined.  The item with the smallest (first)
// priority is always the next item to be accessed or removed by
// Min and RemoveMin.

#include <apvector.h>

template<class ItemType, class PriorityType>  // Used for implementation
struct PQPair{                                // Not for direct use by 
  ItemType     item;                          // the client program
  PriorityType pri;
};

template<class ItemType, class PriorityType>
class PriorityQueue{
  public:

    PriorityQueue();                   // default constructor
      // copy constructor
    PriorityQueue(const PriorityQueue<ItemType, PriorityType> & pq); 

      // assignment operator
    const PriorityQueue<ItemType, PriorityType> & 
      operator = (const PriorityQueue<ItemType, PriorityType> & rhs); 

    // accessors

    const ItemType & Min();            // returns the first priority item
    bool  IsEmpty();                   // returns true if the pq is empty
                                       // otherwise returns false
    // modifiers

    void RemoveMin();                  // removes the first priority item
    void RemoveMin(ItemType & item);   // removes and returns the first 
                                       // priority item
    void Add(const ItemType & item,
             const PriorityType & pri);// adds item to the priority queue

  private:

    apvector<PQPair<ItemType, PriorityType> > myHeap;
    int                                       myNumItems;
};

#include "priorityqueue.cpp"